package fr.smallcrew.security.domain;

import com.google.common.base.Objects;
import com.google.common.base.Strings;
import org.springframework.data.jpa.domain.AbstractPersistable;
import org.springframework.security.authentication.encoding.PasswordEncoder;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.validation.constraints.NotNull;

import static com.google.common.base.Objects.toStringHelper;
import static com.google.common.base.Preconditions.checkNotNull;

@Entity
public class DomainUser extends AbstractPersistable<Integer> {
  private static final long serialVersionUID = -2624558074839859986L;
  public static final String ID = "id";
  public static final String USERNAME = "username";
  public static final String PASSWORD = "password";

  @NotNull
  @Column(length = 32, unique = true)
  private String username;

  @NotNull
  @Column(length = 512)
  private String password;

  @NotNull
  @Column(length = 48)
  private String passwordSalt;

  public DomainUser() {
  }

  public DomainUser(String username, String password) {
    this.username = username;
    this.password = password;
  }

  public String getUsername() {
    return username;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public String getPasswordSalt() {
    return passwordSalt;
  }

  public void setPasswordSalt(String passwordSalt) {
    this.passwordSalt = passwordSalt;
  }

  @Override
  public String toString() {
    return toStringHelper(this)
      .add(ID, getId())
      .add(USERNAME, username)
      .toString();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null || !(obj instanceof DomainUser)) {
      return false;
    }
    DomainUser that = (DomainUser)obj;
    return Objects.equal(getId(), that.getId()) &&
      Objects.equal(username, that.username) &&
      Objects.equal(password, that.password) &&
      Objects.equal(passwordSalt, that.passwordSalt);
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(getId(), password, passwordSalt, username);
  }

  public static class Builder {
    private PasswordEncoder passwordEncoder;
    private String userName;
    private String password;
    private String passwordSalt;

    public Builder userName(String email) {
      this.userName = email;
      return this;
    }

    public Builder password(String password) {
      this.password = password;
      return this;
    }

    public Builder passwordEncoder(PasswordEncoder passwordEncoder) {
      this.passwordEncoder = passwordEncoder;
      return this;
    }

    public Builder passwordSalt(String passwordSalt) {
      this.passwordSalt = passwordSalt;
      return this;
    }

    public DomainUser build() {
      checkNotNull(passwordEncoder, "Password encoder is not set");
      checkNotNull(passwordSalt, "Password salt is not set");

      if (Strings.isNullOrEmpty(userName)) {
        throw new InvalidUsernameException();
      }

      if (Strings.isNullOrEmpty(password) || password.length() < 6) {
        throw new InvalidPasswordException();
      }

      password = password.trim();

      DomainUser user = new DomainUser();
      user.setUsername(userName);
      user.setPassword(passwordEncoder.encodePassword(password, passwordSalt));
      user.setPasswordSalt(passwordSalt);
      return user;
    }
  }
}
